
From: Alasdair G Kergon <agk@redhat.com>

Fix dm_swap_table() __bind error cases: a missing unlock, and EINVAL
preferable to EPERM.

Signed-off-by: Alasdair G Kergon <agk@redhat.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/md/dm.c |   13 +++++--------
 1 files changed, 5 insertions(+), 8 deletions(-)

diff -puN drivers/md/dm.c~device-mapper-fix-dm_swap_table-error-cases drivers/md/dm.c
--- 25/drivers/md/dm.c~device-mapper-fix-dm_swap_table-error-cases	Fri Jul  8 16:45:34 2005
+++ 25-akpm/drivers/md/dm.c	Fri Jul  8 16:45:34 2005
@@ -966,23 +966,20 @@ static void __flush_deferred_io(struct m
  */
 int dm_swap_table(struct mapped_device *md, struct dm_table *table)
 {
-	int r;
+	int r = -EINVAL;
 
 	down_write(&md->lock);
 
 	/* device must be suspended */
-	if (!test_bit(DMF_SUSPENDED, &md->flags)) {
-		up_write(&md->lock);
-		return -EPERM;
-	}
+	if (!test_bit(DMF_SUSPENDED, &md->flags))
+		goto out;
 
 	__unbind(md);
 	r = __bind(md, table);
-	if (r)
-		return r;
 
+out:
 	up_write(&md->lock);
-	return 0;
+	return r;
 }
 
 /*
_
